var Ssp = function()
{
	var ssp = {};
	var debug = 0;
	var manuallySelected = [];
	var selectionIsValid = false;
	var rawDataType;
	var pipeline;
	var annotationTypePipeline;
	var models;
	var numModelsSelected = 0;
	
	// Page initialization
	ssp.initPage = function()
	{
		// Step 1
		Buttons.addClickHandler('btnSelectRawBioAssays', ssp.selectRawBioAssays);
		Events.addEventHandler('rawBioAssays', 'base-selected', ssp.setRawBioAssayCallback);
		Events.addEventHandler('rawBioAssays', 'change', ssp.rawBioAssaysOnChange);
		Events.addEventHandler('step-1', 'wizard-validate', ssp.validateStep1);

		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', ssp.initializeStep2);
		Events.addEventHandler('presets', 'change', ssp.presetOnChange);
		Events.addEventHandler('step-2', 'wizard-validate', ssp.validateStep2);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
			
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', ssp.submit);

		Wizard.showLoadingAnimation('Loading bioassays...');
		var url = '../Ssp.servlet?ID='+App.getSessionId();
		url += '&cmd=GetRawBioAssaysForSsp';
		Wizard.asyncJsonRequest(url, ssp.initializeStep1);
	}
	
	// --- Step 1 -----------------------------------
	ssp.initializeStep1 = function(response)
	{
		var frm = document.forms['reggie'];
		
		rawDataType = response.rawDataType;
		pipeline = response.pipeline;
		
		var rawBioAssays = response.rawBioAssays;
		if (rawBioAssays.length > 0)
		{
			var firstPlate = null;
			for (var rawNo=0; rawNo < rawBioAssays.length; rawNo++)
			{
				var raw = rawBioAssays[rawNo];
				
				var libPlate = raw.lib.bioWell ? raw.lib.bioWell.bioPlate : null;
				if (firstPlate == null && libPlate != null) firstPlate = libPlate.name;
				var option = ssp.createListOption(rawNo+1, raw, raw.DO_NOT_USE == null && libPlate != null && libPlate.name==firstPlate)
				frm.rawBioAssays.options[frm.rawBioAssays.length] = option;
			}
			Events.sendChangeEvent('rawBioAssays');
		}
		else
		{
			Wizard.setInputStatus('rawBioAssays', 'invalid', 'No raw bioassays available for processing');
		}

		Doc.show('step-1');
		Doc.show('gonext');
	}
	
	ssp.validateStep1 = function(event)
	{
		if (!selectionIsValid) event.preventDefault();
	}

	ssp.selectRawBioAssays = function()
	{
		var frm = document.forms['reggie'];
		if (frm.rawBioAssays.disabled) return;
		// Reset list
		manuallySelected = [];
		var url = '&resetTemporary=1';
		if (rawDataType) url += '&tmpfilter:STRING:rawDataType='+encodeURIComponent(rawDataType);
		if (pipeline != null)
		{
			if (annotationTypePipeline == null)
			{
				annotationTypePipeline = Reggie.getAnnotationTypeInfo('PIPELINE');
			}
			url += '&tmpfilter:STRING:'+encodeURIComponent('#')+annotationTypePipeline.id+'='+encodeURIComponent(pipeline);
		}
		Dialogs.selectItem('RAWBIOASSAY', 'rawBioAssays', 1, url);
	}

	ssp.setRawBioAssayCallback = function(event)
	{
		var raw = event.detail;
		
		var opt = Reggie.getListOption('rawBioAssays', raw.id);
		if (opt)
		{
			opt.selected = true;
		}
		else
		{
			manuallySelected[manuallySelected.length] = raw.id;
		}
		
		if (raw.remaining == 0 && manuallySelected.length > 0)
		{
			var url = '../Ssp.servlet?ID='+App.getSessionId();
			url += '&cmd=GetRawBioAssaysForSsp';
			url += '&items='+manuallySelected.join(',');
			Wizard.showLoadingAnimation('Loading bioassays...');
			Wizard.asyncJsonRequest(url, ssp.manuallySelected);
		}
		else
		{
			Events.sendChangeEvent('rawBioAssays');
		}
	}

	ssp.manuallySelected = function(response)
	{
		var rawBioAssays = response.rawBioAssays;
		var frm = document.forms['reggie'];	

		if (rawBioAssays != null && rawBioAssays.length > 0)
		{
			var offset = frm.rawBioAssays.length+1;
			for (var rawNo=0; rawNo < rawBioAssays.length; rawNo++)
			{
				var raw = rawBioAssays[rawNo];
				var option = ssp.createListOption(rawNo+offset, raw, raw.DO_NOT_USE == null);
				frm.rawBioAssays.options[frm.rawBioAssays.length] = option;
			}
			Events.sendChangeEvent('rawBioAssays');
		}
	}
	
	ssp.createListOption = function(index, rawItem, selected)
	{
		var lib = rawItem.lib;
		var isYellow = lib.specimen && lib.specimen.YellowLabel != null;
		var libPlate = lib.bioWell ? lib.bioWell.bioPlate : null;
		
		var tooltip = null;
		var name = (index) + ': ';
		if (libPlate) name += libPlate.name + ' - ';
		name += rawItem.name;
		
		if (rawItem.DO_NOT_USE)
		{
			name += ' [DoNotUse]';
			tooltip = 'DoNotUse-'+Strings.encodeTags(rawItem.DO_NOT_USE+': '+rawItem.DO_NOT_USE_COMMENT);
		}
		else if (rawItem.AutoProcess == 'ReProcess')
		{
			name += ' [R]';
		}
		
		var option = new Option(name, rawItem.id, false, selected);
		if (isYellow) option.className = 'yellow';
		if (tooltip) option.title = tooltip;
		option.rawBioAssay = rawItem;
		return option;
	}

	ssp.rawBioAssaysOnChange = function()
	{
		var frm = document.forms['reggie'];
		
		selectionIsValid = false;
		var numSelected = 0;
		var numDoNotUse = 0;
		var invalidPipeline = null;

		Wizard.setInputStatus('rawBioAssays');
		Wizard.hideGoNextConfirmation();

		for (var rawNo = 0; rawNo < frm.rawBioAssays.length; rawNo++)
		{
			if (frm.rawBioAssays[rawNo].selected) 
			{
				numSelected++;
				
				var raw = frm.rawBioAssays[rawNo].rawBioAssay;
				if (raw.DO_NOT_USE) numDoNotUse++;
				
				if (pipeline && raw.pipeline && raw.pipeline != pipeline)
				{
					invalidPipeline = Strings.encodeTags(raw.name+' ('+raw.pipeline)+') is not intended for the '+pipeline+' pipeline.';
				}
			}
		}
		if (numSelected == 0)
		{
			Wizard.setInputStatus('rawBioAssays', 'invalid', 'Select at least one item');
			return;
		}
		if (invalidPipeline) 
		{
			Wizard.setInputStatus('rawBioAssays', 'invalid', invalidPipeline);
			return;
		}
		selectionIsValid = true;
		
		if (numDoNotUse > 0)
		{
			Wizard.setInputStatus('rawBioAssays', 'warning', numDoNotUse + ' selected items marked as DoNotUse');
			Wizard.showGoNextConfirmation(true, 'Confirm ' + numDoNotUse + ' items marked as DoNotUse');
		}
		else
		{
			Wizard.setInputStatus('rawBioAssays', 'valid');
		}
	}
	
	ssp.initializeStep2 = function()
	{
		Wizard.showLoadingAnimation('Loading bioassays...');
		var url = '../Ssp.servlet?ID='+App.getSessionId();
		url += '&cmd=GetModels';
		Wizard.asyncJsonRequest(url, ssp.modelsLoaded);
	}

	ssp.modelsLoaded = function(response)
	{
		var frm = document.forms['reggie'];

		models = response.models;
		if (models.length == 0)
		{
			Wizard.setFatalError('No SSP models have been configured in reggie-config.xml.');
		}
		
		var presets = response.presets;
		for (var pNo = 0; pNo < presets.length; pNo++)
		{
			var preset = presets[pNo];
			var option = new Option(preset.name);
			option.preset = preset;
			frm.presets[frm.presets.length] = option;
		}
		
		var html = '';
		for (var mNo = 0; mNo < models.length; mNo++)
		{
			var model = models[mNo];
			html += '<tr class="align-top" id="modelrow.'+mNo+'">';
			html += '<td class="subprompt"><label for="model.'+mNo+'">'+Strings.encodeTags(model.name)+'</label></td>';
			html += '<td style="width: 2em;"><input type="checkbox" name="model.'+mNo+'" id="model.'+mNo+'"></td>';
			html += '<td class="input"><div class="modeldescription">'+(Strings.encodeTags(model.description) || '&nbsp;')+'</div></td>';
			html += '<td class="status" id="model.'+mNo+'.status"></td>';
			html += '<td class="help"><span id="model.'+mNo+'.message" class="message"></span></td>';
			html += '</tr>';
		}
		Doc.element('models').innerHTML = html;
		Forms.linkCheckboxesWithLabels(Doc.element('models'));
		
		for (var mNo = 0; mNo < models.length; mNo++)
		{
			var model = models[mNo];
			var checkbox = Doc.element('model.'+mNo);
			if (model.error)
			{
				Wizard.setInputStatus('model.'+mNo, 'warning', Reggie.softLineBreaks(model.error));
				checkbox.disabled = true;
			}
			else
			{
				checkbox.checked = true;
			}
			Events.addEventHandler(checkbox, 'change', ssp.onModelSelected);
		}
		ssp.onModelSelected();
		
		Wizard.setCurrentStep(2);
		Doc.show('goregister');
		Doc.show('gocancel');
		
	}
	
	ssp.presetOnChange = function()
	{
		var frm = document.forms['reggie'];
		var preset = frm.presets[frm.presets.selectedIndex].preset;
		
		for (var mNo = 0; mNo < models.length; mNo++)
		{
			var model = models[mNo];
			if (!frm['model.'+mNo].disabled)
			{
				frm['model.'+mNo].checked = !preset || preset.models.indexOf(model.name) >= 0;
			}
		}
		ssp.onModelSelected();
	}
	
	ssp.onModelSelected = function()
	{
		var frm = document.forms['reggie'];
		numModelsSelected = 0;
		for (var mNo = 0; mNo < models.length; mNo++)
		{
			if (frm['model.'+mNo].checked) numModelsSelected++;
			Doc.addOrRemoveClass('modelrow.'+mNo, 'selected', frm['model.'+mNo].checked);
		}
		
		if (numModelsSelected > 0)
		{
			Wizard.setInputStatus('models', 'valid');
			Doc.removeClass('models', 'invalid');
		}
		else
		{
			Wizard.setInputStatus('models', 'invalid', 'Select at least one model!');
			Doc.addClass('models', 'invalid');
		}
	}
	
	ssp.validateStep2 = function(event)
	{
		if (numModelsSelected == 0) event.preventDefault();
	}
	
	ssp.submit = function()
	{
		var frm = document.forms['reggie'];
		var submitInfo = {};
		
		var selectedRba = [];
		var selectedModels = [];
		submitInfo.rawBioAssays = selectedRba;
		submitInfo.models = selectedModels;
			
		for (var rawNo = 0; rawNo < frm.rawBioAssays.length; rawNo++)
		{
			if (frm.rawBioAssays[rawNo].selected) 
			{
				selectedRba[selectedRba.length] = frm.rawBioAssays[rawNo].rawBioAssay.id;
			}
		}
		for (var mNo = 0; mNo < models.length; mNo++)
		{
			if (frm['model.'+mNo].checked)
			{
				selectedModels[selectedModels.length] = models[mNo].name;
			}
		}

		var url = '../Ssp.servlet?ID='+App.getSessionId();
		url += '&cmd=RunSsp';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, ssp.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	ssp.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}
	
	
	return ssp;
}();

Doc.onLoad(Ssp.initPage);

