var SelectPlates = function()
{
	var select = {};
	var plates;
	var currentPreviewPlateId;
	
	var debug = 0;

	// Page initialization
	select.initPage = function()
	{
		Buttons.addClickHandler('close', App.closeWindow);
		Buttons.addClickHandler('btnOk', select.saveSelected);
		
		Events.addEventHandler('plate-list', 'click', select.previewPlate);
		
		var url = '../Mips.servlet?ID='+App.getSessionId();
		url += '&cmd=GetAliquotPlatesForMips';
		var ignore = Data.get('page-data', 'ignore');
		if (ignore) url += '&ignore='+encodeURIComponent(ignore);
		Wizard.showLoadingAnimation('Loading information about aliquot plates...');
		Wizard.asyncJsonRequest(url, select.aliquotPlatesLoaded);		
	}
	
	select.aliquotPlatesLoaded = function(response)
	{
		plates = response.plates;
		if (plates.length == 0)
		{
			Wizard.setFatalError('Could not find any aliquot plates');
			return;
		}
		
		var html = '';
		for (var plateNo = 0; plateNo < plates.length; plateNo++)
		{
			var plate = plates[plateNo];
			
			var notes = [];
			if (plate.PlateProcessing == 'AddAliquots') notes[notes.length] = 'SCAN-B lab';
			if (plate.AutoProcessing == 'ReProcess') notes[notes.length] = 'Re-process';
			if (plate.comments) notes[notes.length] = Strings.encodeTags(plate.comments);
			html += '<tr class="highlight">';
			html += '<td class="check"><input type="checkbox" id="check.'+plate.id+'" value="'+plate.id+'"></td>';
			html += '<td class="name"><label for="check.'+plate.id+'" data-plate-id="'+plate.id+'">'+Strings.encodeTags(plate.name)+'</label></td>';
			html += '<td class="image"><img class="link" src="../images/search.png" data-plate-id="'+plate.id+'" title="Preview aliquots on this plate"></td>';
			html += '<td class="count">'+plate.usedWells+'</td>';
			html += '<td class="count">'+plate.unusedAliquots+'</td>';
			html += '<td class="date">'+Reggie.reformatDate(plate.registrationDate)+'</td>';
			html += '<td class="notes">'+notes.join('; ')+'</td>';
			html += '</tr>';
		}
		
		Doc.element('plate-list').innerHTML = html;
		Doc.show('aliquot-table', 'table');
	}

	select.previewPlate = function(event)
	{
		var plateId = Data.get(event.target, 'plate-id');
		if (!plateId || plateId == currentPreviewPlateId) return;
		
		currentPreviewPlateId = plateId;
		
		var url = '../Mips.servlet?ID='+App.getSessionId();
		url += '&cmd=GetUnusedAliquotsFromPlates';
		url += '&plates='+plateId;
		url += '&includeall=1';
		Wizard.asyncJsonRequest(url, select.previewLoaded);
	}
	
	select.previewLoaded = function(response)
	{
		var plate = response.plates[0];
		var html = '<table class="fullborder">';
		html += '<caption>Preview of ' + Strings.encodeTags(plate.name)+'</caption>';
		html += '<tr class="bg-filled-100 bottomborder"><th></th>';
		for (var c = 0; c < plate.columns; c++)
		{
			html += '<th>'+(c+1)+'</th>';
		}
		html += '</tr>';
		
		for (var r = 0; r < plate.rows; r++)
		{
			var row = Reggie.wellToAlpha(r);
			html += '<tr><th class="bg-filled-100 rightborder">'+row+'</th>';
			for (var c = 0; c < plate.columns; c++)
			{
				var id = row+(c+1);
				html += '<td id="'+id+'"></td>';
			}
			html += '</tr>';
		}
		var panelCounters = [];
		html += '<tr class="topborder pool-row">';
		html += '<th class="bg-filled-100 rightborder"></th>';
		for (var c = 0; c < plate.columns; c++)
		{
			html += '<th><div id="c.'+c+'"></div></th>';
			panelCounters[c] = new ItemCounter();
		}
		html += '</tr>';
		html += '</table>';

		Doc.element('plate-preview').innerHTML = html;
		
		for (var a = 0; a < plate.aliquots.length; a++)
		{
			var aliquot = plate.aliquots[a];
			var td = Doc.element(aliquot.bioWell.location);
			var cls = 'aliquot ' + MipsColor.getColor(aliquot.MIPS_Panel);
			if (aliquot.numChildren > 0) cls += ' used';
			td.className = cls;
			td.title = aliquot.MIPS_Panel;
			panelCounters[aliquot.bioWell.column].count(aliquot.MIPS_Panel);
		}
		for (var c = 0; c < plate.columns; c++)
		{
			counter = panelCounters[c];
			var mipsPanel = Strings.encodeTags(counter.maxKey);
			if (mipsPanel)
			{
				var numPanels = counter.numKeys();
				var div = Doc.element('c.'+c);
				div.innerHTML = mipsPanel + (numPanels > 1 ? '+' : '');
				div.title = mipsPanel + (numPanels > 1 ? ' and ' + (numPanels- 1) + ' more' : '');
				Doc.addClass(div.parentNode, MipsColor.getColor(mipsPanel));
			}
		}
	}
	
	select.saveSelected = function()
	{
		var frm = document.forms['reggie'];
		
		var selected = [];
		for (var plateNo = 0; plateNo < plates.length; plateNo++)
		{
			var plate = plates[plateNo];
			if (frm['check.'+plate.id].checked)
			{
				selected[selected.length] = {'id': plate.id, 'name': plate.name };
			};
		}
		if (selected.length > 0)
		{
			var notifyTarget = window.opener.Doc.element('plates');
			var remaining = selected.length-1;
			for (var i = 0; i < selected.length; i++)
			{
				var s = selected[i];
				s.remaining = remaining--;
				Events.sendCustomEvent(notifyTarget, 'base-selected', s);
			}
		}
		App.closeWindow();
	}
	
	return select;
}();

Doc.onLoad(SelectPlates.initPage);

